"""Filters."""
from __future__ import unicode_literals


class Filter(object):
    """Base filter class."""

    def __init__(self, options):
        """Initialization."""

    def filter(self, text, encoding):
        """Execute filter."""

        return text
